@@ -5,12 +5,11 @@
  *
  * Copyright (C) 2005-2006  NTT DATA CORPORATION
  *
- * Version: 1.1.2   2006/06/02
+ * Version: 1.1   2006/04/01
  *
- * This program saves policies from /proc/ccs/policy/ directory
- * to /root/security/ directory.
- * You can call this program to save policies just before
- * the system halts (i.e. at the end of "halt" script).
+ * This program saves policies in /proc/ccs/policy/ directory.
+ * You can call this program to save policies just before the system halts
+ * (i.e. at the end of "halt" script).
  * This program remounts root fs as read/write if necessary.
  *
  */
@@ -19,6 +18,9 @@
 #include <sys/mount.h>
 #include <time.h>
 
+DOMAIN_INFO *domain_list[4] = { NULL, NULL, NULL, NULL };
+int domain_list_count[4] = { 0, 0, 0, 0 };
+
 static char *ReadFile(const char *filename) {
 	char *buffer = NULL;
 	int fd;
@@ -42,39 +44,6 @@ static char *ReadFile(const char *filename) {
 	return buffer;
 }
 
-static int IsSameDomainList(void) {
-	if (domain_list_count[0] == domain_list_count[1]) {
-		int i, j;
-		for (i = 0; i < domain_list_count[0]; i++) {
-			const char **string_ptr = domain_list[0][i].string_ptr;
-			const int string_count = domain_list[0][i].string_count;
-			if (string_count == domain_list[1][i].string_count) {
-				const char **ptr = domain_list[1][i].string_ptr;
-				for (j = 0; j < string_count; j++) {
-					/* Faster comparison, for they are SaveName'd and sorted pointers. */
-					if (string_ptr[j] != ptr[j]) break;
-				}
-				if (j == string_count) continue;
-			}
-			break;
-		}
-		if (i == domain_list_count[0]) return 1;
-	}
-	return 0;
-}
-
-static void ClearDomainPolicy(const int type) {
-	int index;
-	for (index = 0; index < domain_list_count[type]; index++) {
-		free(domain_list[type][index].string_ptr);
-		domain_list[type][index].string_ptr = NULL;
-		domain_list[type][index].string_count = 0;
-	}
-	free(domain_list[type]);
-	domain_list[type] = NULL;
-	domain_list_count[type]= 0;
-}
-
 int main(int argc, char *argv[]) {
 	int remount_root = 0;
 	static char filename[1024];
@@ -92,7 +61,7 @@ int main(int argc, char *argv[]) {
 		return 0;
 	}
 	if (argc == 1) {
-		force_save = save_system_policy = save_exception_policy = save_domain_policy = 1;
+		force_save = save_domain_policy = 1;
 	} else {
 		int i;
 		for (i = 1; i < argc; i++) {
@@ -113,7 +82,7 @@ int main(int argc, char *argv[]) {
 					   "d : Save domain_policy.\n"
 					   "a : Save all policies.\n"
 					   "f : Save even if on-disk policy and on-memory policy are the same.\n\n"
-					   "If no options given, this program assumes 'a' and 'f' are given.\n", argv[0]);
+					   "If no options given, this program assumes 'd' and 'f' are given.\n", argv[0]);
 				return 0;
 			}
 		}
@@ -130,11 +99,6 @@ int main(int argc, char *argv[]) {
 		remount_root = 1;
 	}
 
-	/* Exclude nonexistent policy. */
-	if (access("/proc/ccs/policy/system_policy", R_OK)) save_system_policy = 0;
-	if (access("/proc/ccs/policy/exception_policy", R_OK)) save_exception_policy = 0;
-	if (access("/proc/ccs/policy/domain_policy", R_OK)) save_domain_policy = 0;
-
 	/* Repeat twice so that necessary permissions for this program are included in domain policy. */
 	for (repeat = 0; repeat < 2; repeat++) {
 
@@ -144,8 +108,7 @@ int main(int argc, char *argv[]) {
 			if (new_policy && (force_save || !old_policy || strcmp(new_policy, old_policy))) {
 				int fd;
 				snprintf(filename, sizeof(filename) - 1, "system_policy.%02d-%02d-%02d.%02d:%02d:%02d.txt", tm->tm_year % 100, tm->tm_mon + 1, tm->tm_mday, tm->tm_hour, tm->tm_min, tm->tm_sec);
-				if ((fd = open(filename, O_WRONLY | O_CREAT, 0600)) != EOF) {
-					ftruncate(fd, 0);
+				if ((fd = open(filename, O_WRONLY | O_CREAT | O_TRUNC, 0600)) != EOF) {
 					write(fd, new_policy, strlen(new_policy));
 					close(fd);
 					unlink("system_policy.txt");
@@ -164,8 +127,7 @@ int main(int argc, char *argv[]) {
 			if (new_policy && (force_save || !old_policy || strcmp(new_policy, old_policy))) {
 				int fd;
 				snprintf(filename, sizeof(filename) - 1, "exception_policy.%02d-%02d-%02d.%02d:%02d:%02d.txt", tm->tm_year % 100, tm->tm_mon + 1, tm->tm_mday, tm->tm_hour, tm->tm_min, tm->tm_sec);
-				if ((fd = open(filename, O_WRONLY | O_CREAT, 0600)) != EOF) {
-					ftruncate(fd, 0);
+				if ((fd = open(filename, O_WRONLY | O_CREAT | O_TRUNC, 0600)) != EOF) {
 					write(fd, new_policy, strlen(new_policy));
 					close(fd);
 					unlink("exception_policy.txt");
@@ -177,21 +139,17 @@ int main(int argc, char *argv[]) {
 			free(old_policy);
 			free(new_policy);
 		}
-
-	}
-	
-	if (save_domain_policy) {
-		ReadDomainPolicy("/proc/ccs/policy/domain_policy", 0);
-		for (repeat = 0; repeat < 10; repeat++) {
-			//if (repeat) printf("Domain policy has changed while saving domain policy. Retrying.\n");
-			if (access("domain_policy.txt", R_OK) == 0) ReadDomainPolicy("domain_policy.txt", 1);
-			/* Need to save domain policy? */
-			if (force_save || !IsSameDomainList()) {
+		
+		if (save_domain_policy) {
+			const int src = repeat ? 2 : 0;
+			const int dest = repeat ? 3 : 1;
+			ReadDomainPolicy("/proc/ccs/policy/domain_policy", src);
+			ReadDomainPolicy("domain_policy.txt", dest);
+			if (force_save || !IsSameDomainList(src, dest)) {
 				int fd;
 				snprintf(filename, sizeof(filename) - 1, "domain_policy.%02d-%02d-%02d.%02d:%02d:%02d.txt", tm->tm_year % 100, tm->tm_mon + 1, tm->tm_mday, tm->tm_hour, tm->tm_min, tm->tm_sec);
-				if ((fd = open(filename, O_WRONLY | O_CREAT, 0600)) != EOF) {
-					ftruncate(fd, 0);
-					WriteDomainPolicy(fd, 0);
+				if ((fd = open(filename, O_WRONLY | O_CREAT | O_TRUNC, 0600)) != EOF) {
+					WriteDomainPolicy(fd, src);
 					close(fd);
 					unlink("domain_policy.txt");
 					symlink(filename, "domain_policy.txt");
@@ -199,14 +157,7 @@ int main(int argc, char *argv[]) {
 					printf("Can't create %s\n", filename);
 				}
 			}
-			/* Has domain policy changed while saving domain policy? */
-			ClearDomainPolicy(0);
-			ReadDomainPolicy("/proc/ccs/policy/domain_policy", 0);
-			if (IsSameDomainList()) break;
-			ClearDomainPolicy(1);
 		}
-		ClearDomainPolicy(0);
-		ClearDomainPolicy(1);
 	}
 	if (remount_root) mount("/", "/", "rootfs", MS_REMOUNT | MS_RDONLY, NULL);
 	return 0;
